'use strict';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var Promise$1 = require("bluebird");

var _require = require("lodash");
var flatMap = _require.flatMap;

var addic7ed = require("./sources/addic7ed");
var opensubtitles = require("./sources/opensubtitles");

var CaptionCore = function () {
  function CaptionCore() {
    classCallCheck(this, CaptionCore);

    this.sources = [addic7ed, opensubtitles];
  }

  createClass(CaptionCore, [{
    key: "searchByQuery",
    value: function searchByQuery(query) {
      var language = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "eng";
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 10;

      var checkSources = this.sources.map(function (source) {
        return source.textSearch(query, language, limit);
      });

      return {
        on: function on(event, callback) {
          switch (event) {
            case "fastest":
              // Wait for first source to finish downloading, return first set of results to renderer.
              Promise$1.any(checkSources).then(function (results) {
                callback(results);
              });

              return this;

            case "completed":
            default:
              // Check whether all sources have been checked. Return to renderer and hide loading spinner.
              Promise$1.all(checkSources).then(function (results) {
                var emptyResultSet = results.filter(function (result) {
                  return result.length > 0;
                });

                if (emptyResultSet.length === 0) {
                  callback([]);
                }

                callback(flatMap(results));
              });

              return this;
          }
        }
      };
    }
  }, {
    key: "searchByFiles",
    value: function searchByFiles() {
      var files = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var language = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "eng";
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 10;

      var opensubtitlesRef = opensubtitles.fileSearch(files, language, limit);

      return {
        on: function on(event, callback) {
          switch (event) {
            case "completed":
            default:
              // First promise which is resolved should return its results
              Promise$1.race([opensubtitlesRef]).then(function (results) {
                return callback(results);
              });

              return this;
          }
        }
      };
    }
  }]);
  return CaptionCore;
}();

module.exports = new CaptionCore();
